
/*
Lithobox2 is a program created to generate 3D printable lithophanes

copyright 2018 Jennifer Weiler
http://jenweiart.com
////////////////////////////////   */
float steps_distFromTop = 35;
float steps_orbSize = 30;
float steps_innerOrbSize = steps_orbSize-5;
float steps_inBetween = 150;
float steps_center = 450;
color selected = color(255, 255, 255);
color not_selected = color(107, 191, 236);
color not_selected_hover = color(177, 230, 245);

void showSteps(int cStep) { 
  blendMode(BLEND);
  ellipseMode(CENTER);
  textAlign(CENTER);
  noStroke();

  stepOrb(1, steps_center-steps_inBetween, steps_distFromTop+steps_orbSize, cStep, "3D shape");
  stepOrb(2, steps_center, steps_distFromTop+steps_orbSize, cStep, "texture");
  stepOrb(3, steps_center+ steps_inBetween, steps_distFromTop+steps_orbSize, cStep, "size");

  arrow(steps_center-(steps_inBetween/2), steps_distFromTop+steps_orbSize);
  arrow(steps_center+(steps_inBetween/2), steps_distFromTop+steps_orbSize);
}

void stepOrb(int t, float transX, float transY, int cStep, String txt) {

  color c;
  if (cStep == t) {  
    c = selected;
  } else if (PVector.dist(new PVector(transX, transY), new PVector(mouseX, mouseY)) <  steps_orbSize) { 
    c =not_selected_hover;
  } else {  
    c =not_selected;
  }
  fill(c);
  ellipse(transX, transY, steps_orbSize*2, steps_orbSize*2);
  fill(backgroundColor);
  ellipse(transX, transY, steps_innerOrbSize*2, steps_innerOrbSize*2);
  fill(c);
  textSize(steps_innerOrbSize*2);
  text(t, transX, transY+(steps_innerOrbSize*.7));
  textSize(steps_innerOrbSize*.5);
  text(txt, transX, transY-(steps_innerOrbSize*1.5));
}

void arrow(float transX, float transY) {
  // pushMatrix();
  // translate(transX, transY);
  strokeWeight(steps_orbSize - steps_innerOrbSize);
  stroke(not_selected);
  line(transX-20, transY, transX, transY);
  line(transX, transY-15, transX+15, transY);
  line(transX, transY+15, transX+15, transY);
  // popMatrix();
}



/////////////////////////
/////////////////////////

void step0() {
  textAlign(CENTER);
  textSize(steps_innerOrbSize*2);
  fill(255);
  textSize(32);
  text("Welcome to LithoBox", width/2, 250);
  textSize(16);
  text("Here, you will be able to make\n 3D printable-lithophanes in 3 easy steps", width/2, 300);
  for (int i=0; i<stepButtons_0.size(); i++) {
    stepButtons_0.get(i).render(true);
  }
}
/////////////////////////
/////////////////////////
PImage drawingBox; // dimensions 3x4
PImage drawnBoxSave = new PImage(100, 100);
float dB_x = 450;
float dB_y = 120;
float dB_wid = 450;
float dB_hei = 300;
Boolean drawnInBox = false;


void step1() {
  for (int i=0; i<stepButtons_1.size(); i++) {
    stepButtons_1.get(i).render(true);
  }
  image(drawingBox, dB_x, dB_y, dB_wid, dB_hei);
  pushMatrix();
  translate(width, 0);
  scale(-1, 1);
  image(drawingBox, dB_x, dB_y, dB_wid, dB_hei);
  popMatrix();
  fill(200);
  text("Templates", 100, 450);

  // fill(200);
  textSize(steps_innerOrbSize*.5);
  text("(.gif, .jpg, .tga, or .png)", 435, 565);

  instructionBox();
}

/////////////////////
  
float whiteX;//  = (23.0 /  vaseHEIGHT_inches.getNumberMilimeters()) * drawingBox.height;/// 23 mm scaled to hightcv
////////////////////

float radiusOfBottom = 1.25; //inches
Boolean checkThicknessDraw(int x, int y) {
  if (y<100) {
    float t = map(y, 0, 100,  0, whiteX);
    if ( x<t  ) {
      return true;
    } else {
      return false;
    }
  } else if (y<(drawingBox.height-50)) {
    if (x<whiteX) {
      return true;
    } else {
      return false;
    }
  } else {
    if (x<((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())-(drawingBox.height-y)) {
      return true;
    } else {
      return false;
    }
  }
}

Boolean checkBaseOuterThickness(int x, int y) {
  if (y>=(drawingBox.height-50)) {
    if (x>1+((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())+(2*(drawingBox.height-y))) {
      return false;
    }
  }

  return true;
}

void checkThickness_uploadedImage() {        

  
  drawingBox.loadPixels();
  for (int y=0; y<drawingBox.height; y++) {
    if (y<100) {
      for (int x=0; x<=(y/3); x++) {
        if ( x<100) {
          int loc = (x) + (y)*drawingBox.width;
          drawingBox.pixels[loc] =  color(255);
        }
      }
    } else if (y<(drawingBox.height-50)) {  
      for (int x=0; x<=whiteX; x++) {
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  color(255);
      }
      
      
      
    } else {
      for (int x=0; x<((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())-(drawingBox.height-y); x++) {
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  color(255);
      }
      for (int x=int(1+((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())+(2*(drawingBox.height-y))); x>drawingBox.width; x++) {
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  color(0);
      }
      
    
      
  
        
        
    }
  }
  drawingBox.updatePixels();
}


void checkThickness_drawnImage() {
  
  int xForClosestPartOfVaseResize = -1;
  drawingBox.loadPixels();
  for (int y=0; y<drawingBox.height; y++) {
    if (y<100) {
      for (int x=0; x<=30; x++) {
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  color(255);
      }
    } else if (y<(drawingBox.height-50)) {
      for (int x=0; x<whiteX; x++) {
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  color(255);
      }
      if(STAGE == 3 && y==(drawingBox.height-51)){
        int litX = 0;
        while(xForClosestPartOfVaseResize == -1 && litX < drawingBox.width){
          int loc = (litX) + (y)*drawingBox.width;
          float col = red(drawingBox.pixels[loc]);
          if(col < 127){ xForClosestPartOfVaseResize = litX; }
          litX++;
        }
        if(xForClosestPartOfVaseResize == -1){xForClosestPartOfVaseResize = drawingBox.width-1;}
      }
    } else {
      float b = ((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())-(drawingBox.height-y);
      if (b >= drawingBox.width) { 
        b =drawingBox.width;
      }
      for (int x=0; x<b; x++) {
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  color(255);
      }
      if (drawingBox.width > int(1+((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())+(2*(drawingBox.height-y)))) {
        for (int x=int(1+((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())+(2*(drawingBox.height-y))); x<drawingBox.width; x++) {
          int loc = (x) + (y)*drawingBox.width;
          drawingBox.pixels[loc] =  color(0);
        }
      }

      if (STAGE == 3) {
        int whiteBorder = int((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())-(drawingBox.height-y);
        int blackBorder = int(1+((radiusOfBottom*drawingBox.height)/vaseHEIGHT_inches.getNumberInches())+(2*(drawingBox.height-y)));
        if (whiteBorder < drawingBox.width) {
          if (blackBorder >= drawingBox.width) { 
            blackBorder = drawingBox.width;
          }
          for (int x=whiteBorder; x<blackBorder; x++) {
            int loc = (x) + (y)*drawingBox.width;
            if(x < xForClosestPartOfVaseResize){
            drawingBox.pixels[loc] =  color(255);
            }else{
            drawingBox.pixels[loc] =  color(0);  
            }
          }
        }
      }
    }
  }
  drawingBox.updatePixels();
}


void proportionDrawingBox() {
  drawnInBox = false;

  PImage back = drawingBox.get();
  drawingBox.resize(int( float(drawingBox.height) * (dB_wid/dB_hei)), drawingBox.height);
  
  whiteX  = (9.0 /  vaseHEIGHT_inches.getNumberMilimeters()) * drawingBox.height;
  println(vaseHEIGHT_inches.getNumberMilimeters());

  drawingBox.loadPixels();
  back.loadPixels();


  // determine if left size is black (assuming 2 sided vase)
  // or white (assuming 1 sided vase)
  int sidePos = drawingBox.width * 50;
  float average = (red(drawingBox.pixels[sidePos])+green(drawingBox.pixels[sidePos])+blue(drawingBox.pixels[sidePos]))/3;
  // println(average);
  if (average > 127) {  // if WHITE
    boolean negativeSpace = false;

    for (int y = 0; y < drawingBox.height; y++) {
      for (int x = 0; x < drawingBox.width; x++) {
        color c;
        if (x<back.width && negativeSpace == false) {
          int bacloc = (x) + (y)*back.width;
          float r = red(back.pixels[bacloc]);
          float g = green(back.pixels[bacloc]);
          float b = blue(back.pixels[bacloc]);
          float a = (r+g+b)/3;
          if (a<(255/2)) { 
            a=0;
            negativeSpace = true;
          } else {  
            a=255;
          }
          c = color(a);
        } else {
          c = color(0);
        }
        int loc = (x) + (y)*drawingBox.width;
        drawingBox.pixels[loc] =  c;
      }
      negativeSpace = false;
    }
  } else {  // if BLACK

    // determine if point at opposite side is white
    sidePos = drawingBox.width-1;
    average = (red(drawingBox.pixels[sidePos])+green(drawingBox.pixels[sidePos])+blue(drawingBox.pixels[sidePos]))/3;

    if (average > 127) {


      boolean negativeSpace = false;

      for (int y = 0; y < drawingBox.height; y++) {
        for (int x = 0; x < drawingBox.width; x++) {
          color c;
          if (x<back.width && negativeSpace == false) {
            int bacloc = (back.width-1-x) + (y)*back.width;
            float r = red(back.pixels[bacloc]);
            float g = green(back.pixels[bacloc]);
            float b = blue(back.pixels[bacloc]);
            float a = (r+g+b)/3;
            if (a<(255/2)) { 
              a=0;
              negativeSpace = true;
            } else {  
              a=255;
            }
            c = color(a);
          } else {
            c = color(0);
          }
          int loc = (x) + (y)*drawingBox.width;
          drawingBox.pixels[loc] =  c;
        }
        negativeSpace = false;
      }
    } else { // we're assuming whole vase, symmetrical, is being displayed


      boolean negativeSpace = false;

      for (int y = 0; y < drawingBox.height; y++) {
        for (int x = 0; x < drawingBox.width; x++) {
          color c;
          if (x<(back.width/2) && negativeSpace == false) {
            int bacloc = (x+(back.width/2)) + (y)*back.width;
            float r = red(back.pixels[bacloc]);
            float g = green(back.pixels[bacloc]);
            float b = blue(back.pixels[bacloc]);
            float a = (r+g+b)/3;
            if (a<(255/2)) { 
              a=0;
              negativeSpace = true;
            } else {  
              a=255;
            }
            c = color(a);
          } else {
            c = color(0);
          }
          int loc = (x) + (y)*drawingBox.width;
          drawingBox.pixels[loc] =  c;
        }
        negativeSpace = false;
      }
    }
  }


  drawingBox.updatePixels();
  drawingBox.resize(int( float(600) * (dB_wid/dB_hei)), 600);
  checkThickness_uploadedImage();
  //drawingBox.save("ss.jpg");
}

void drawInBox1() {
  float xDist = abs(mouseX - dB_x);
  float yDist = mouseY - dB_y;
  int xPos  = int( float(drawingBox.width) * (xDist / dB_wid));
  int yPos  = int( float(drawingBox.height) * (yDist / dB_hei));
  if (xPos >=0 && xPos < drawingBox.width && yPos >=0 && yPos < drawingBox.height) {
    drawnInBox = true;
    changesMade = true;
    int yWidth = int(8*  float(drawingBox.width)/dB_wid);
    int upperY = yPos -yWidth;
    if (upperY < 0) { 
      upperY = 0;
    }
    int lowerY = yPos +yWidth;
    if (lowerY >= drawingBox.height) { 
      lowerY = drawingBox.height-1;
    }

    drawingBox.loadPixels();
    for (int y=upperY; y<lowerY; y++) {
      // int curntY;
      for (int x=0; x<drawingBox.width; x++) {
        int loc = (x) + (y)*drawingBox.width;
        color c;
        if ((x <= (xPos-abs((y-((lowerY+upperY)/2))/4)) || (checkThicknessDraw(x, y))) && checkBaseOuterThickness(x, y)) {
          c = color(255);
        } else {
          c = color(0);
        }
        drawingBox.pixels[loc] =  c;
      }
    }
    drawingBox.updatePixels();
  }
}
/////////////////////////
/////////////////////////
PImage step2ExampleSrcImg;
PImage step2ExampleVaseImg;
Boolean changesMade = true;
Boolean outerImageWhite = true;
Boolean innerImageWhite = true;

void step2() {
  drawPreview(.003);
  for (int i=0; i<stepButtons_2.size(); i++) {
    boolean b = true;
    if (i==1 && !outerImageWhite) { 
      b = false;
    } else if (i==2 && !innerImageWhite) { 
      b = false;
    }
    stepButtons_2.get(i).render(b);
  }
  fill(200);
  text("Selected images must be in \n format .gif, .jpg, .tga, or .png", 150, 300);
  image(step2ExampleSrcImg, 105, 350, 90, 45);
  // strokeWeight(steps_orbSize - steps_innerOrbSize);
  // stroke(not_selected);
  float transX = 150;
  float transY = 430;
  line(transX, transY-20, transX, transY);
  line(transX, transY+15, transX-15, transY);
  line(transX, transY+15, transX+15, transY);
  image(step2ExampleVaseImg, 0, 460);

  instructionBox();
}

void updatePreviewVase() {
  PGraphics p = createGraphics(250, 100, P2D);
  PImage srcTexture = step2ExampleSrcImg.get();
  srcTexture.resize(200, 100);
  PImage srcShape = drawingBox.get();
  srcShape.resize(150, 100);
  srcShape.loadPixels();

  p.beginDraw();
  p.noStroke();
  p.fill(255);
  for (int y=0; y<p.height; y++) {
    int w = 0;
    for (int x=0; x<srcShape.width; x++) {
      int loc = x + y*srcShape.width;
      if (red(srcShape.pixels[loc]) < 127) {
        w = x;
        break;
      }
    }

    float left = ((srcShape.width-w));
    float right = ((srcShape.width-w)+(w*2)) ;


    p.beginShape();
    p.texture(srcTexture);
    p.vertex( left, y, 50, y);
    p.vertex(  right, y, 150, y);
    p.vertex( left, y+1, 50, y+1);
    p.endShape();  

    p.beginShape();
    p.texture(srcTexture);
    p.vertex(  right, y, 150, y);
    p.vertex( left, y+1, 50, y+1);
    p.vertex( right, y+1, 150, y+1);
    p.endShape();
  }
  p.endDraw();
  step2ExampleVaseImg = p.get();
  // step2ExampleVaseImg.save("ddd.png");
}

/////////////////////////
/////////////////////////

Boolean saveScreenUp = false;
String saveStep = "";
int delay = 0;

boolean changesMadeStep3 = false;
void step3() {


  float wP = 180;
  float hP = 180;
  line(wP, 150, 450-(outline_width)/2, 150);
  line(wP, 150+outline_height, 450-(outline_width)/2, 150+outline_height);
  line(wP, 150, wP, 270);
  line(wP, 150+outline_height, wP, 360);
  line(450-(outline_width)/2, 150+outline_height, 450-(outline_width)/2, hP+outline_height);
  line(450+(outline_width)/2, 150+outline_height, 450+(outline_width)/2, hP+outline_height);
  line(450-(outline_width)/2, hP+outline_height, 400, hP+outline_height);
  line(450+(outline_width)/2, hP+outline_height, 500, hP+outline_height);
  if(changesMadeStep3) {
      
      checkThickness_drawnImage();
       loadPreview();
       changesMadeStep3 = false;
  
  }
  drawPreview(0.0);
  vaseHEIGHT_inches.render();
  vaseWIDTH_inches.render();


  for (int i=0; i<stepButtons_3.size(); i++) {
    stepButtons_3.get(i).render(true);
  }

  if (saveScreenUp) {
    rect(200, 150, width-400, height-300);
    fill(selected);
    textSize(30);
    if (saveStep == "saved") {
      text("SAVED!", width/2, 200);
      fill(backgroundColor);
      textSize(15);
      text("Your file has been exported as: "+mostRecentName+".obj", width/2, 260);
      text("You can either exit now or continue to make changes", width/2, 300);
      for (int i=0; i<saveButtons.size(); i++) {
        saveButtons.get(i).render(true);
      }
    } else if (saveStep == "saving") {
      text("SAVING...", width/2, 200);
      delay++;
    }
  }

  instructionBox();
}


void displayScaler() {
}

/////////////////////////
/////////////////////////

void fileSelected(File selection) {
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
  } else {
    if (STAGE == 2) {
      if (stepButtons_2.get(1).clicked == true) {
        stepButtons_2.get(1).clicked = false;
        String ending = (selection.getAbsolutePath()).substring(selection.getAbsolutePath().length()-4);
        if (ending.equals(".jpg") || ending.equals("jpeg") || ending.equals(".JPG") || ending.equals(".gif") ||  ending.equals(".tga") || ending.equals(".png") ) {
          println("User selected " + selection.getAbsolutePath());
          img1 = selection.getAbsolutePath();
          newOuterPreviewImg = true;
          outerImageWhite = false;
          loadPreview();
        }
      } else if (stepButtons_2.get(2).clicked == true) {
        stepButtons_2.get(2).clicked = false;
        String ending = (selection.getAbsolutePath()).substring(selection.getAbsolutePath().length()-4);
        if (ending.equals(".jpg") || ending.equals("jpeg") || ending.equals(".JPG") || ending.equals(".gif") ||  ending.equals(".tga") || ending.equals(".png") ) {
          println("User selected " + selection.getAbsolutePath());
          img2 = selection.getAbsolutePath();
          newInnerPreviewImg = true;
          innerImageWhite = false;
          loadPreview();
        }
      }
    } else if (STAGE==1 && stepButtons_1.get(5).clicked == true) {
      stepButtons_1.get(5).clicked = false;
      String ending = (selection.getAbsolutePath()).substring(selection.getAbsolutePath().length()-4);
      if (ending.equals(".jpg") || ending.equals("jpeg") ||  ending.equals(".JPG") || ending.equals(".gif") ||  ending.equals(".tga") || ending.equals(".png") ) {
        println("User selected " + selection.getAbsolutePath());
        // figure out how to turn image into example image
        if (drawnInBox) {  
          drawnBoxSave = drawingBox.get();  
          set4thBox();
        }
        drawingBox = loadImage(selection.getAbsolutePath());
        // drawingBox = greyscale(drawingBox);
        proportionDrawingBox();
        changesMade = true;
    
      }
    }
  }
}

//////////////////////////

////instruction box

String instruction1  = "Select shape of lithophane. Shapes can also be drawn in program by clicking and dragging on the image.";
String instruction2 = "Select inner and outer image to be displayed on lithophane. Inner image will appear blurrier and will not be visible when there is not a light source inside the 3D model.";
String instruction3 = "Size model for 3D printing. The intended size of the model will affect how thick the walls of the 3D model will be.";

void instructionBox() {

  String s;
  if (STAGE == 1) {
    s = instruction1;
  } else if (STAGE == 2) {
    s = instruction2;
  } else {
    s = instruction3;
  }

  fill(255);
  textSize(14);
  text(s, width*36/48, 25, 200, 300);
}
