
/*
Lithobox2 is a program created to generate 3D printable lithophanes

copyright 2018 Jennifer Weiler
http://jenweiart.com
////////////////////////////////   */
boolean newOuterPreviewImg = true;
boolean newInnerPreviewImg = true;


float outline_height;
float outline_width;

void loadPreview() {

  if (newOuterPreviewImg) {
    outerPreview = loadImage(img1);
    outerSurface = outerPreview.get(); 
 
    outerPreview.resize( int(previewResolution*(float(outerPreview.width)/float(outerPreview.height))), previewResolution);
    outerPreview = greyscale(outerPreview);
    newOuterPreviewImg= false;
  }
  if (newInnerPreviewImg) {
    innerPreview = loadImage(img2);
    innerSurface = innerPreview.get();
     
    innerPreview.resize( int(previewResolution*(float(innerPreview.width)/float(innerPreview.height))), previewResolution);
    int iw = innerPreview.width;
    int ih = innerPreview.height;
    int scaleDown = 4;
    innerPreview.resize(innerPreview.width/scaleDown, innerPreview.height/scaleDown);
    innerPreview.resize(iw, ih);
    innerPreview = greyscale(innerPreview);
    newInnerPreviewImg = false;
  }

  outlinePreview = new ArrayList<PVector>();

  PImage src;
  if (drawingBox == null) {
    src = loadImage(image_name);
  } else {
    src = drawingBox.get();
  }
  src.loadPixels(); 

  int rows = src.width;
  int cols = src.height;

  int scaleRes = int ( float(cols) / float(previewResolution) );


  outline_height = float(cols) / float(scaleRes);
  outline_width = 0;


  ////// GENERATE POINTS FOR OUTER VASE
  //  int actualCols_outer = 0;
  int previous = -1; // -1 if no data, 0 if black (less than 127), 1 if white (more than 127)
  for (int pic_y = 0; pic_y < cols; pic_y+=scaleRes) {
    for (int pic_x = 0; pic_x < rows; pic_x++) {

      int src_loc = (pic_x) + (pic_y)*(src.width);

      float r = red(src.pixels[src_loc]);
      float g = green(src.pixels[src_loc]);
      float b = blue(src.pixels[src_loc]);

      float c = (r+g+b)/(255*3); // gets to greyscale between 0 and 1

      if (c<.5 && previous == 1) {
        // used to - generate ring based on pic_x as radius
        outlinePreview.add(new PVector(float(pic_x)/float(scaleRes), float(pic_y)/float(scaleRes)));
        if(2*(float(pic_x)/float(scaleRes)) > outline_width){
            outline_width = 2*(float(pic_x)/float(scaleRes));
        }
      }

      if (c>.5) { 
        previous = 1;
      } else { 
        previous = 0;
      }
    }
    previous = -1;
  }
  
  setWIDTHbasedOnHEIGHT(false);
}


PImage greyscale(PImage p) {

  p.loadPixels();
  for (int y = 0; y < p.height; y++) {
    for (int x = 0; x < p.width; x++) {
      int loc = (x) + (y)*p.width;
      float r = red(p.pixels[loc]);
      float g = green(p.pixels[loc]);
      float b = blue(p.pixels[loc]);
      float a = floor((r+g+b)/3);
      color c = color(a, a, a);
      p.pixels[loc] =  c;
    }
  }
  p.updatePixels();
  return p;
}






void drawPreviewSection(ArrayList<PVector> oP, float a, float b, PImage img, float startPos, float endPos) {

  for (int i=0; i<oP.size()-1; i++) {  
    float left = map(a, 0, 1, -oP.get(i).x, oP.get(i).x);
    float right = map(b, 0, 1, -oP.get(i).x, oP.get(i).x);

    beginShape();
    texture(img);
    vertex( left, oP.get(i).y, startPos, img.height*(float(i)/float(oP.size())));
    vertex(  right, oP.get(i).y, endPos, img.height*(float(i)/float(oP.size())));
    vertex( left, oP.get(i+1).y, startPos, img.height*(float(i+1)/float(oP.size())));
    endShape();  

    beginShape();
    texture(img);
    vertex(  right, oP.get(i).y, endPos, img.height*(float(i)/float(outlinePreview.size())));
    vertex( left, oP.get(i+1).y, startPos, img.height*(float(i+1)/float(outlinePreview.size())));
    vertex( right, oP.get(i+1).y, endPos, img.height*(float(i+1)/float(outlinePreview.size())));
    endShape();
  }
}



//////////////////////////////////////////////////////////////////
int previewHeight = 300;
int previewResolution = 300;
float scaler = float(previewHeight) / float(previewResolution);

ArrayList<PVector> outlinePreview;
PImage outerPreview;
PImage innerPreview;

float rot_position = 0;
//float rotSpeed = .003;

//////////////////////////////////////////////////////////////////

void drawPreview(float rotSpeed) {



  pushMatrix();
  translate(450, 150);



  noStroke();

  scale(scaler, scaler);

  fill(20);
  blendMode(SUBTRACT);
  float bottomY = outlinePreview.get(outlinePreview.size()-1).y;
  beginShape();
  for (int i=0; i<outlinePreview.size(); i++) {
    vertex( outlinePreview.get(i).x+(outlinePreview.size()-i), ((bottomY*2)+outlinePreview.get(i).y)/3);
  }
  for (int i=outlinePreview.size()-1; i>=0; i--) {
    vertex( (outlinePreview.size()-i)-outlinePreview.get(i).x, ((bottomY*2)+outlinePreview.get(i).y)/3);
  }
  endShape(); 

  fill(230);
  blendMode(ADD);
  beginShape();
  for (int i=0; i<outlinePreview.size(); i++) {
    vertex( outlinePreview.get(i).x, outlinePreview.get(i).y);
  }
  for (int i=outlinePreview.size()-1; i>=0; i--) {
    vertex( -outlinePreview.get(i).x, outlinePreview.get(i).y);
  }
  endShape();  

  blendMode(MULTIPLY);


  float startPos = rot_position * innerPreview.width;
  float endPos = rot_position + .5;
  if (endPos >= 1) {  
    endPos -= 1;
  }
  endPos = endPos * innerPreview.width;

  // inner
  if (startPos < endPos) {
    drawPreviewSection(outlinePreview, 0, 1, innerPreview, startPos, endPos);
  } else {
    float d = ((innerPreview.width)-(2*(startPos-(innerPreview.width/2)))) / float(innerPreview.width); 
    drawPreviewSection(outlinePreview, 0, d, innerPreview, startPos, innerPreview.width);
    drawPreviewSection(outlinePreview, d, 1, innerPreview, 0, endPos);
  }


  startPos = rot_position * outerPreview.width;
  endPos = rot_position + .5;
  if (endPos >= 1) {  
    endPos -= 1;
  }
  endPos = endPos * outerPreview.width;

  // outer
  if (startPos < endPos) {
    drawPreviewSection(outlinePreview, 0, 1, outerPreview, startPos, endPos);
  } else {
    float d = ((outerPreview.width)-(2*(startPos-(outerPreview.width/2)))) / float(outerPreview.width); 
    drawPreviewSection(outlinePreview, 0, d, outerPreview, startPos, outerPreview.width);
    drawPreviewSection(outlinePreview, d, 1, outerPreview, 0, endPos);
  }



  popMatrix();


  rot_position += rotSpeed;
  if (rot_position >= 1) {  
    rot_position -= 1;
  }
  blendMode(BLEND);
}
