
/*
Lithobox2 is a program created to generate 3D printable lithophanes

copyright 2018 Jennifer Weiler
http://jenweiart.com
////////////////////////////////   */
boolean saveOBJ = true;
PrintWriter output;
int ptNum = 1;

PImage pImg; 
String image_name = "test3.png"; // side view

Boolean blendVase = true;

PImage outerSurface;
PImage innerSurface;
String img1 = "nocutout.png"; // outer image
String img2 = "nocutout.png"; // inner image

/// need 2 because they will be resized differently for outer and inner
PImage cutOutImage_out;
PImage cutOutImage_in;
String img3 = "nocutout.png";
Vase v;

String mostRecentName = "";

String folderName = "";


PGraphics sideVisualizer; // (for beta testing wall thickness consistency)
boolean sideVisTurnedOn = false;

void printVase(String name) {
  mostRecentName = name;
  float horizontal = 600; // horizontal and vertical should be divisible by 10
  float vertical = 600;
  
  if(sideVisTurnedOn){sideVisualizer = createGraphics(int(horizontal), int(vertical*2));}

  int surfaceSides = 600;
  
  // defaults, these values are changed later further down
  float depthOfSurfaceOuter = 5;
  float depthOfSurfaceInner = 7; //5
  float thickness = .2; // full thickness //3
  float scaler = 1;

  pImg = drawingBox.get();
  float ratio = float(pImg.width) / float(pImg.height);

  if (ratio > 1) {
    vertical = horizontal / ratio;
  } else {
    horizontal = vertical * ratio;
  }
  int segLength = 2;
  pImg.resize(int(horizontal/segLength), int(vertical/segLength));
  
  
  float print_Height = 1;
  float scalerFormat = 1;
  if(!scaleExportInMilimeters){
    print_Height = vaseHEIGHT_inches.getNumberInches();
  }else{
    print_Height = vaseHEIGHT_inches.getNumberMilimeters();
    scalerFormat = 25.4;
  }
  //------------------------------------------
  //------------------------------------------
  // set scaler, thickness, and depth of Surface outer and inner
  float minimizer = .07;
  float minimizer_inner = .12;
  scaler = print_Height /  pImg.height;
  thickness = scalerFormat * (.04 *.3) * (float(pImg.height) / print_Height);
  depthOfSurfaceOuter = scalerFormat * minimizer * (float(pImg.height) / print_Height);
  depthOfSurfaceInner = scalerFormat * minimizer_inner  * (float(pImg.height) / print_Height);
  
 // println(scaler+" "+thickness+" "+depthOfSurfaceOuter);
  //------------------------------------------
  //------------------------------------------
 // outerSurface = outerPreview.get();  //loadImage(img2)
 // innerSurface = innerPreview.get(); //loadImage(img2);
  cutOutImage_out = loadImage(img3);
  cutOutImage_in = loadImage(img3);
  
  outerSurface.save(folderName+"outer image.png");
  innerSurface.save(folderName+"inner image.png");

  output = createWriter(folderName+name+".obj");

  output.println("# Blender3D v249 OBJ File:");
  output.println("# www.blender3d.org");
  output.println("mtllib texture_circ.mtl");


  v = new Vase(pImg, outerSurface, innerSurface, cutOutImage_out, cutOutImage_in, surfaceSides, depthOfSurfaceOuter, depthOfSurfaceInner, thickness, scaler);

  output.flush();
  output.close();
  saveOBJ = false;
  
  drawingBox.save(folderName+name+".png");
  
  ptNum = 1;
}

class Vase {
  float x;
  float y;
  float z;
  float x_length;
  float y_length;
  float z_length;

  int [][] relief_outer;
  int [][] relief_inner;

  ArrayList<PVector> outerPts;
  ArrayList<PVector> innerPts;

  int cols = 0;
  int rows = 0;

  // number of vector points in each ring of vase
  int sides;
  float surfaceDepth_outer;
  float surfaceDepth_inner;
  float THICKNESS;
  float SCALER;

  Vase(PImage img, PImage img_outer, PImage img_inner, PImage cut_out, PImage cut_in, int s, float d1, float d2, float t, float sca) {

    outerPts = new ArrayList<PVector>();
    innerPts = new ArrayList<PVector>();

    sides = s;
    surfaceDepth_outer = d1;
    surfaceDepth_inner = d2;
    THICKNESS = t;
    SCALER = sca;
    relief_mesh(img, img_outer, img_inner, cut_out, cut_in);
    
    
    
  
  }




  void relief_mesh(PImage src, PImage img_outer, PImage img_inner, PImage cut_out, PImage cut_in) {
   // println(SCALER);
    src.loadPixels(); 

    rows = src.width;
    cols = src.height;
    //  bas_relief = new float[rows][cols];

    ////// GENERATE POINTS FOR OUTER VASE
    //  int actualCols_outer = 0;
    int previous = -1; // -1 if no data, 0 if black (less than 127), 1 if white (more than 127)
    for (int pic_y = 0; pic_y < cols; pic_y++) {
      for (int pic_x = 0; pic_x < rows; pic_x++) {

        int src_loc = (pic_x) + (pic_y)*(src.width);

        float r = red(src.pixels[src_loc]);
        float g = green(src.pixels[src_loc]);
        float b = blue(src.pixels[src_loc]);

        float c = (r+g+b)/(255*3); // gets to greyscale between 0 and 1

        if (c<.5 && previous == 1) {
          // generate ring based on pic_x as radius
          outerPts.add(new PVector(pic_x, pic_y));// + THICKNESS
          //actualCols_inner++;
        }

        if (c>.5) { 
          previous = 1;
        } else { 
          previous = 0;
        }

        /* if (c>.5 && previous == 0) {
         // used to - generate ring based on pic_x as radius
         outerPts.add(new PVector(pic_x + THICKNESS, pic_y));
         //  actualCols_outer++;
         }
         
         if (c>.5) { 
         previous = 1;
         } else { 
         previous = 0;
         }//*/
      }
      previous = -1;
    }

    // println("outerPoints" + outerPts);
    if (blendVase) {
      outerPts = GaussianBlur1D(outerPts);
    }

    ////// GENERATE OUTER POINTS (SURFACE RING)
    relief_outer = new int[sides][outerPts.size()];
    img_outer.resize(  sides, outerPts.size());
    img_outer.loadPixels();

    cut_out.resize(sides, outerPts.size());
    cut_out.loadPixels();


    for (int k=0; k<outerPts.size(); k++) {
      float px = outerPts.get(k).x;
      float py = outerPts.get(k).y;
      
      PVector surfaceThick = new PVector(0, 1); 

      if (k>0 && k<outerPts.size()-1) {
        PVector o2 = PVector.sub(new PVector(outerPts.get(k-1).x, outerPts.get(k-1).y), new PVector(outerPts.get(k+1).x, outerPts.get(k+1).y));
        float an= o2.heading();
          surfaceThick.rotate(an);
      } else if (k==0) {
        PVector o2 = PVector.sub(new PVector(outerPts.get(k).x, outerPts.get(k).y), new PVector(outerPts.get(k+1).x, outerPts.get(k+1).y));
        float an= o2.heading();
          surfaceThick.rotate(an);
      } else { // k == outerPts.size()-1
        PVector o2 = PVector.sub(new PVector(outerPts.get(k-1).x, outerPts.get(k-1).y), new PVector(outerPts.get(k).x, outerPts.get(k).y));
        float an= o2.heading();
          surfaceThick.rotate(an);
      }

      
      
                   ///////////////////////////////////////////////////////////////////////////////////////////////
      ///////////////////////////////////////////////////////////////////////////////////////////////
      ///////////////////////////////////////////////////////////////////////////////////////////////
      
        if(sideVisTurnedOn){
          
          surfaceThick.mult(100);
          PVector p2= PVector.add(surfaceThick, outerPts.get(k));
          
          float scala = 4;
      sideVisualizer.beginDraw();
 
    sideVisualizer.stroke(255);
    sideVisualizer.strokeWeight(2);
     sideVisualizer.line(outerPts.get(k).x*scala, outerPts.get(k).y*scala, p2.x*scala,p2.y*scala);
      sideVisualizer.endDraw();
        }
      
      
      ///////////////////////////////////////////////////////////////////////////////////////////////
      ///////////////////////////////////////////////////////////////////////////////////////////////
     
      
      
      
      /// println(surfaceThick);
      float angle = 360 / float(sides);
      for (int i = 0; i < sides; i++) {
        int src_loc = (k*sides)+i;

        float cutR = red(cut_out.pixels[src_loc]);
        float cutG = green(cut_out.pixels[src_loc]);
        float cutB = blue(cut_out.pixels[src_loc]);

        if (!(cutR>200 && cutG<50 && cutB<50)) { // if red

          float r = red(img_outer.pixels[src_loc]);
          float g = green(img_outer.pixels[src_loc]);
          float b = blue(img_outer.pixels[src_loc]);

          float c = 1.0- ((r+g+b)/(255*3)); // gets to greyscale between 0 and 1

          PVector sT = new PVector(surfaceThick.x, surfaceThick.y);
          sT.setMag((c*surfaceDepth_outer) + THICKNESS);
          float px2 = (px + sT.x);
          float py2 = py + sT.y;
          
          if(py2 > outerPts.get(outerPts.size()-1).y){
              py2 = outerPts.get(outerPts.size()-1).y; 
          }

          float x = cos( radians( i * angle ) ) * px2;
          float z = sin( radians( i * angle ) ) * px2;
          PVector e = warp(new PVector(x, py2, z), true);
          output.println("v "+(e.x*SCALER)+" "+(e.y*SCALER)+" "+(e.z*SCALER));
          relief_outer[i][k] = ptNum;
          ptNum++;
        } else {
          relief_outer[i][k] = -1;
        }
      }
    }
    img_outer.updatePixels();
    cut_out.updatePixels();


    ////// CONNECTING SIDES FOR OUTER CYLINDER
    for (int t_y = 0; t_y < outerPts.size(); t_y++) {
      for (int t_x = 0; t_x < sides; t_x++) {
        int curnt = relief_outer[t_x][t_y];
        if (curnt != -1) {
          if (t_y>0) {
            int up1 = relief_outer[t_x][t_y-1];
            int left1;
            if (t_x>0) {
              left1 = relief_outer[t_x-1][t_y];
            } else {
              left1 = relief_outer[sides-1][t_y];
            }
            if (up1 != -1 && left1 != -1) {
              output.println("f "+curnt+" "+up1+" "+left1);
            }
          }
          if (t_y<outerPts.size()-1) {
            int down1 = relief_outer[t_x][t_y+1];
            int right1;
            if (t_x<sides-1) {
              right1 = relief_outer[t_x+1][t_y];
            } else {
              right1 = relief_outer[0][t_y];
            }
            if (down1 != -1 && right1 != -1) {
              output.println("f "+curnt+" "+down1+" "+right1);
            }
          }
        }
      }
    }//*/
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    //int insidePtNum = ptNum;
    Boolean hollowTop = true;
    Boolean hollowBottom = true;
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////

    ////// GENERATE POINTS FOR INNER VASE
    //int actualCols_inner = 0;
    IntList innerBreaks = new IntList(); 
    previous = -1; // -1 if no data, 0 if black (less than 127), 1 if white (more than 127)
    Boolean hollowLine = false;
    for (int pic_y = 0; pic_y < cols; pic_y++) {
      for (int pic_x = 0; pic_x < rows; pic_x++) {

        int src_loc = ((pic_x)) + ((pic_y))*(src.width);
        //int src_loc = ((pic_x*segLength)) + ((pic_y*segLength))*(src.width);

        float r = red(src.pixels[src_loc]);
        float g = green(src.pixels[src_loc]);
        float b = blue(src.pixels[src_loc]);

        float c = (r+g+b)/(255*3); // gets to greyscale between 0 and 1

        if (c<.5 && previous == 1) {
          // generate ring based on pic_x as radius
          innerPts.add(new PVector(pic_x, pic_y));//- THICKNESS
          //actualCols_inner++;
          hollowLine = true;
        }

        if (c>.5) { 
          previous = 1;
        } else { 
          previous = 0;
        }

      }
      previous = -1;
      if (hollowLine == false) {
        if (pic_y == 0) { 
          hollowTop = false;
        } else if (pic_y == cols-1) { 
          hollowBottom = false;
        }


        if (innerBreaks.size() == 0 || innerBreaks.get(innerBreaks.size()-1) != innerPts.size()) {
          innerBreaks.append(innerPts.size()-1);
        }
      }
      hollowLine = false;
      
      /// move innerpoint based on position of outerPoints
      ////////////////////////////////////////////////////////////////////////////////////////
      /////////////////////////////////////////////////////////////////////////////////
      /////////////////////////////////////////////////////////////////////////////////
      /////////////////////////////////////////////////////////////////////////////////
      
      int t=innerPts.size()-1;
      float px = outerPts.get(t).x;
      float py = outerPts.get(t).y;

      /// rotate vector so it points away from bowl
      PVector upP = new PVector(0, 0);
      PVector downP = new PVector(0, 0);
      if (t>0) {
       // upP = new PVector(outerPts.get(t-1).x, outerPts.get(t-1).y);
        upP = PVector.sub(upP, new PVector(px, py));
      }
      if (t<outerPts.size()-1) {
       // downP = new PVector(outerPts.get(t+1).x, outerPts.get(t+1).y);
        downP = PVector.sub(downP, new PVector(px, py));
      }

      PVector surfaceThick = new PVector(0,1); 

      if (t>0 && t<outerPts.size()-1) {
      PVector o2 = PVector.sub(new PVector(outerPts.get(t+1).x, outerPts.get(t+1).y), new PVector(outerPts.get(t-1).x, outerPts.get(t-1).y));
        float an= o2.heading();
               surfaceThick.rotate(an);
             
      } else if (t==0) {
        PVector o2 = PVector.sub(new PVector(outerPts.get(t+1).x, outerPts.get(t+1).y), new PVector(outerPts.get(t).x, outerPts.get(t).y));
        float an= o2.heading();
        surfaceThick = new PVector(0,1); 
          surfaceThick.rotate(an);
      } else if(t==outerPts.size()-1) { // k == outerPts.size()-1
        PVector o2 = PVector.sub(new PVector(outerPts.get(t).x, outerPts.get(t).y), new PVector(outerPts.get(t-1).x, outerPts.get(t-1).y));
        float an= o2.heading();
        surfaceThick.rotate(an);
      }
      float dist = SCALER * THICKNESS;
      surfaceThick.setMag(dist);
      PVector p = PVector.add(surfaceThick, outerPts.get(t));
     
  
     
      
      
      innerPts.set(t, p);
      

     
      /////////////////////////////////////////////////////////////////////////////////
      /////////////////////////////////////////////////////////////////////////////////
      /////////////////////////////////////////////////////////////////////////////////
      
    }

    //   println("innerPoints" + innerPts);
    if (blendVase) {
      innerPts = GaussianBlur1D(innerPts);
    }

    ///////GENERATE INNER POINTS (SURFACE RING)
    relief_inner = new int[sides][innerPts.size()];
    // img_inner.resize(  sides, innerPts.size());
    img_inner.resize(  sides, outerPts.size());
    img_inner.loadPixels();
    //cut_in.resize(sides, innerPts.size());
    cut_in.resize(sides, outerPts.size());
    cut_in.loadPixels();


    for (int k=0; k<innerPts.size(); k++) {

      
            int t=k;
      float px = outerPts.get(t).x;
      float py = outerPts.get(t).y;

      /// rotate vector so it points away from bowl
    
      PVector surfaceThick = new PVector(0, 1); 

      if (t>0 && t<outerPts.size()-1) {
        PVector o2 = PVector.sub(new PVector(outerPts.get(t+1).x, outerPts.get(t+1).y), new PVector(outerPts.get(t-1).x, outerPts.get(t-1).y));
        float an= o2.heading();
          surfaceThick.rotate(an);
      } else if (t==0) {
        PVector o2 = PVector.sub(new PVector(outerPts.get(t+1).x, outerPts.get(t+1).y), new PVector(outerPts.get(t).x, outerPts.get(t).y));
        float an= o2.heading();
          surfaceThick.rotate(an);
      } else { // k == outerPts.size()-1
        PVector o2 = PVector.sub(new PVector(outerPts.get(t).x, outerPts.get(t).y), new PVector(outerPts.get(t-1).x, outerPts.get(t-1).y));
        float an= o2.heading();
          surfaceThick.rotate(an);
      }

      //*/


      float angle = 360 / float(sides);
      for (int i = 0; i < sides; i++) {
        

        /// try to resize 
        float k2 = float(k)*(float(outerPts.size())/float(innerPts.size()));
        int _k2 = 0;
        if (k2-int(k2)<.5) {
          _k2 = int(k2);
        } else {
          _k2 = int(k2)+1;
        }


        int src_loc = (_k2*sides)+i;


        float cutR = red(cut_in.pixels[src_loc]);
        float cutG = green(cut_in.pixels[src_loc]);
        float cutB = blue(cut_in.pixels[src_loc]);

        if (!(cutR>200 && cutG<50 && cutB<50)) { // if red
          float r = red(img_inner.pixels[src_loc]);
          float g = green(img_inner.pixels[src_loc]);
          float b = blue(img_inner.pixels[src_loc]);

          float c = 1.0 - ((r+g+b)/(255*3)); // gets to greyscale between 0 and 1

          PVector sT = new PVector(surfaceThick.x, surfaceThick.y);
          sT.setMag((c*surfaceDepth_inner) + THICKNESS);
          float px2 = (px + sT.x);
          float py2 = py + sT.y;
          
         

          float x = cos( radians( i * angle ) ) * px2;
          float z = sin( radians( i * angle ) ) * px2;
          PVector e = new PVector(x, py2, z);  //warp(new PVector(x, py2, z), false);
          float s = (sides / 9);
          if(e.y*SCALER >(vaseHEIGHT_inches.getNumberMilimeters()-7) && ((i<s)|| (i>=(s*3) && i<(s*4)) ||  (i>=(s*6) && i<(s*7))   )){
             px2 = 17;
             e.x = cos( radians( i * angle ) ) * px2;
             e.z = sin( radians( i * angle ) ) * px2;

               if(e.y > outerPts.get(outerPts.size()-1).y){
              e.y = outerPts.get(outerPts.size()-1).y - ((outerPts.size()-1-k)*.001); 
          }
          
             output.println("v "+(e.x)+" "+(e.y*SCALER)+" "+(e.z));  
          }else{
            
            if(e.y > outerPts.get(outerPts.size()-1).y){
              e.y = outerPts.get(outerPts.size()-1).y; 
          }
          
          
             output.println("v "+(e.x*SCALER)+" "+(e.y*SCALER)+" "+(e.z*SCALER));  
          }
      
          relief_inner[i][k] = ptNum;
          ptNum++;
        } else {
          relief_inner[i][k] = -1;
        }
      }
    }
    img_inner.updatePixels();
    cut_in.updatePixels();//*/






    ////// CONNECTING FACES FOR INSIDE OF CYLINDER
    for (int t_y = 0; t_y < innerPts.size(); t_y++) {
      for (int t_x = 0; t_x < sides; t_x++) {
        int curnt = relief_inner[t_x][t_y];
        if (curnt != -1) {
          if (t_y>0) {
            int up1 = relief_inner[t_x][t_y-1];
            int left1;
            if (t_x>0) {
              left1 = relief_inner[t_x-1][t_y];
            } else {
              left1 = relief_inner[sides-1][t_y];
            }
            if (up1 != -1 && left1 != -1) {
              output.println("f "+curnt+" "+left1+" "+up1);
            }
          }
          if (t_y<innerPts.size()-1) {
            int down1 = relief_inner[t_x][t_y+1];
            int right1;
            if (t_x<sides-1) {
              right1 = relief_inner[t_x+1][t_y];
            } else {
              right1 = relief_inner[0][t_y];
            }
            if (down1 != -1 && right1 != -1) {
              output.println("f "+curnt+" "+right1+" "+down1);
            }
          }
        }
      }
    }




    ////// TIE OUTER AND INNER AT TOP
    if (!hollowTop) { // if top is solid
      /// inner
      output.print("f ");
      for (int t_x = sides-1; t_x >= 0; t_x--) {
        if (relief_inner[t_x][0] != -1) {
          output.print( (relief_inner[t_x][0]) + " ");
        }
      } 
      output.println(" ");
      // outer
      output.print("f ");
      for (int t_x = 0; t_x < sides; t_x++) {
        if (relief_outer[t_x][0] != -1) {
          output.print( (relief_outer[t_x][0]) + " ");
        }
      }
      output.println(" ");//*/

      boolean completeLoop = false;
      boolean closedOut = true;
      boolean closedIn = true;

      int counter = 0;

      int prev_o = -1;
      int prev_i = -1;
      int land_o = -1;
      int land_i = -1;

      while (!(completeLoop && closedOut && closedIn)) {
        if (closedOut) {
          if (relief_outer[counter][0] == -1 && prev_o != -1) {
            closedOut = false;
          } else if (relief_outer[counter][0] != -1 && land_o == -1) {
            prev_o = relief_outer[counter][0];
          }
        }
        if (closedIn) {
          if (relief_inner[counter][0] == -1 && prev_i != -1) {
            closedIn = false;
          } else if (relief_inner[counter][0] != -1 && land_i == -1) {
            prev_i = relief_inner[counter][0];
          }
        } else if (!closedOut || !closedIn) {
          if (relief_outer[counter][0] != -1) {
            land_o = relief_outer[counter][0];
            closedOut = true;
          }
          if (relief_inner[counter][0] != -1) {
            land_i = relief_inner[counter][0];
            closedIn = true;
          }

          if (closedOut && closedIn) {
            output.println("f "+(prev_o)+" "+(prev_i)+" "+(land_o));
            output.println("f "+(prev_i)+" "+(land_i)+" "+(land_o));
            prev_o = land_o;
            prev_i = land_i;
            land_o = -1;
            land_i = -1;
          }
        }
        counter++;
        if (counter >= sides) {
          counter = 0; 
          completeLoop = true;
        }
      }
    } else { //// connect outer to inner at top
      for (int t_x = 0; t_x < sides; t_x++) {
        int o1 = relief_outer[t_x][0];
        int i1 = relief_inner[t_x][0];
        if (o1 != -1 && i1 != -1) {
          if (t_x<sides-1) {
            int o2 = relief_outer[t_x+1][0];
            int i2 = relief_inner[t_x+1][0];
            if (o2 != -1 && i2 != -1) {
              output.println("f "+(o1)+" "+(o2)+" "+(i1));
              output.println("f "+(i1)+" "+(o2)+" "+(i2));
            }
          } else {
            int o2 = relief_outer[0][0];
            int i2 = relief_inner[0][0];
            if (o2 != -1 && i2 != -1) {
              output.println("f "+(o1)+" "+(o2)+" "+(i1));
              output.println("f "+(i1)+" "+(o2)+" "+(i2));
            }
          }
        }
      }
    }//*/

    ////// TIE OUTER AND INNER AT BOTTOM
    if (!hollowBottom) {

      /// inner
      output.print("f ");
      for (int t_x = 0; t_x < sides; t_x++) {
        if (relief_inner[t_x][innerPts.size()-1] != -1) {
          output.print( (relief_inner[t_x][innerPts.size()-1]) + " ");
        }
      }
      output.println(" ");


      // outer
      output.print("f ");
      for (int t_x = sides-1; t_x >= 0; t_x--) {
        if (relief_outer[t_x][outerPts.size()-1] != -1) {
          output.print( (relief_outer[t_x][outerPts.size()-1]) + " ");
        }
      }
      output.println(" ");//*/


      boolean completeLoop = false;
      boolean closedOut = true;
      boolean closedIn = true;

      int counter = 0;

      int prev_o = -1;
      int prev_i = -1;
      int land_o = -1;
      int land_i = -1;

      while (!(completeLoop && closedOut && closedIn)) {
        if (closedOut) {
          if (relief_outer[counter][outerPts.size()-1] == -1 && prev_o != -1) {
            closedOut = false;
          } else if (relief_outer[counter][outerPts.size()-1] != -1 && land_o == -1) {
            prev_o = relief_outer[counter][outerPts.size()-1];
          }
        }
        if (closedIn) {
          if (relief_inner[counter][innerPts.size()-1] == -1 && prev_i != -1) {
            closedIn = false;
          } else if (relief_inner[counter][innerPts.size()-1] != -1 && land_i == -1) {
            prev_i = relief_inner[counter][innerPts.size()-1];
          }
        } else if (!closedOut || !closedIn) {
          if (relief_outer[counter][outerPts.size()-1] != -1) {
            land_o = relief_outer[counter][outerPts.size()-1];
            closedOut = true;
          }
          if (relief_inner[counter][innerPts.size()-1] != -1) {
            land_i = relief_inner[counter][innerPts.size()-1];
            closedIn = true;
          }

          if (closedOut && closedIn) {
            output.println("f "+(prev_o)+" "+(land_o)+" "+(prev_i));
            output.println("f "+(prev_i)+" "+(land_o)+" "+(land_i));
            prev_o = land_o;
            prev_i = land_i;
            land_o = -1;
            land_i = -1;
          }
        }
        counter++;
        if (counter >= sides) {
          counter = 0; 
          completeLoop = true;
        }
      }
    } else { // connect outer to inner at bottom

      for (int t_x = 0; t_x < sides; t_x++) {
        int o1 = relief_outer[t_x][outerPts.size()-1];
        int i1 = relief_inner[t_x][innerPts.size()-1];
        if (o1 != -1 && i1 != -1) {
          if (t_x<sides-1) {
            int o2 = relief_outer[t_x+1][outerPts.size()-1];
            int i2 = relief_inner[t_x+1][innerPts.size()-1];
            if (o2 != -1 && i2 != -1) {
              output.println("f "+(o1)+" "+(i1)+" "+(o2));
              output.println("f "+(i1)+" "+(i2)+" "+(o2));
            }
          } else {
            int o2 = relief_outer[0][outerPts.size()-1];
            int i2 = relief_inner[0][innerPts.size()-1];
            if (o2 != -1 && i2 != -1) {
              output.println("f "+(o1)+" "+(i1)+" "+(o2));
              output.println("f "+(i1)+" "+(i2)+" "+(o2));
            }
          }
        }
      }
    }//*/



    printSides(outerPts, innerPts, relief_outer, relief_inner, sides);
    if(sideVisTurnedOn){
    

    sideVisualizer.save("visualizer.png");
  }

    src.updatePixels();
  }
}
